/*
  2015-07-26: Created by xopr

  This sketch uses the following external library/libraries
  - https://github.com/milesburton/Arduino-Temperature-Control-Library
*/

/*
 * List of defines that you can use to set up a ESP-01
 * DallasTemperature  onewire sensor network
 * DHT                single DHT temp/humidity sensor
 * NeoPixels          neopixel strand
 * SpacestateSwitch   switch input that sets space state
 * 
 * TMP10x             I2C temperature
 * MCP9808            I2C precision temperature
 * MPL115A2           I2C barometric pressure/temperature
 * MPL3115A2          I2C barometric pressure/altitude/temperature
 * LTC2990            I2C voltage/current/temperature
 */
#define DEBUG


// Switch and pixels are defined on GPIO0 (choose one)
//#define SPACESTATE        // Space state switch
#define NEOPIXELS         // NeoPixels spacestate indicator
#define NEOPIXEL_COUNT 8  // Number of NeoPixels on the connected string

// 'complex' Sensors are on GPIO2 (choose one)
// NOTE: this pin is pulled to GND to activate the programming bootloader)
//#define DALLAS_TEMPERATURE  // Dallas onewire temperature network
//NOT IMPLEMENTED //#define DHT_SENSOR          // DHT temperature and humidity sensor
//NOT IMPLEMENTED //#define DHT_ID [ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ]

// I2C (two wire interface) needs both GPIO0 and GPIO2
//NOT IMPLEMENTED //#define I2C

// Accesspoint definitions
const char* strSsid     = "ACKspaceWifi";
const char* strPassword = "nospacenet"; // Yes it's on the wiki already

// API settings
const char* strServer   = "ackspace.nl";
const char* strApiKey   = "<NO_YOU_CANNOT_HAVE_THE_API_KEY>";

///////////////////////////////////////////////////////////////////////////////////
// NOTE: you progably don't need to edit anything below this point, unless you want to break stuff
///////////////////////////////////////////////////////////////////////////////////
#define PIN_SWITCH    0
#define PIN_NEOPIXELS 2
// NOTE: GPIO2 is also the bootloader mode pin
//#define PIN_ONEWIRE   2

// Sanity check on GPIO0
#if defined( SPACESTATE ) && defined( NEOPIXELS )
    #error Cannot use spacestate switch and neopixels: they use the same pin.
#endif
#if defined( NEOPIXELS ) && !defined( NEOPIXEL_COUNT )
    #error You must define the number of NeoPixels if you want to use them (NEOPIXEL_COUNT).
#endif

// Sanity check on GPIO2
#if defined( DALLAS_TEMPERATURE ) && defined( DHT_SENSOR )
    #error Cannot use Dallas temperature and DHT sensor: they use the same pin.
#endif
#if defined( DHT_SENSOR ) && !defined( DHT_ID )
    #error You must define a custom ID if you want to use DHT (DHT_ID).
#endif

// Sanity check on I2C (GPIO0 and GPIO2)
#if defined( I2C ) && ( defined( SPACESTATE ) || defined( NEOPIXELS ) || defined( DALLAS_TEMPERATURE ) || defined( DHT_SENSOR ) )
    #error Cannot combine I2C (Two Wire Interface) with any other sensors or outputs.
#endif

// Used libraries
#include <ESP8266WiFi.h>

// Forward declarations
void wifiConnect();

#ifdef NEOPIXELS
//#include "JsonVarFetch.h"
#include "StreamJsonReader.h"
#include <Adafruit_NeoPixel.h>
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
Adafruit_NeoPixel pixels = Adafruit_NeoPixel( NEOPIXEL_COUNT, PIN_NEOPIXELS, NEO_GRB + NEO_KHZ800);

// IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
// pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
// and minimize distance between Arduino and first pixel.  Avoid connecting
// on a live circuit...if you must, connect GND first.
#endif

// Dallas temperature uses onewire library
#ifdef DALLAS_TEMPERATURE
#define SENSORS
#define ONEWIRE
#endif

#ifdef ONEWIRE
#include <OneWire.h>
OneWire oneWire( PIN_ONEWIRE );
#endif

// TODO: this is where my file got corrupted
//#ifdef DALLAS_TEMPERATURE

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// setup
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup()
{
    Serial.begin( 115200 );
    delay( 10 );
    
    connectWireless( );

    updateSpacestateIndicator( );

    pinMode( PIN_NEOPIXELS, OUTPUT );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// loop
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop( )
{
    // TODO: IM IN UR LOOP
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// connectWireless
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void connectWireless()
{
    Serial.print( "\nConnecting to " );
    Serial.print( strSsid );

    WiFi.begin( strSsid, strPassword );

    while ( WiFi.status( ) != WL_CONNECTED )
    {
        delay( 500 );
        Serial.print( "." );
    }

    Serial.println( "" );
    Serial.print( "connected; IP:" );
    Serial.println( WiFi.localIP( ) );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// updateSpacestateIndicator
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void updateSpacestateIndicator()
{
#ifdef NEOPIXELS
    uint32_t color;
    color = determineSpaceStateColor();

    for( int i = 0; i < NEOPIXEL_COUNT; i++ )
        pixels.setPixelColor( i, color );

    pixels.show(); // This sends the updated pixel color to the hardware.

#endif
}

#ifdef NEOPIXELS
uint32_t determineSpaceStateColor()
{
    WiFiClient  client;
    char        character;
    uint8_t     nNewLines = 0;
    uint16_t    nPos = 0;

    static const char* queries[] = { "state.open" };
    StreamJsonReader jsonreader( queries, 1 );

    if ( !client.connect( strServer, 80 ) )
    {
        Serial.println("connection failed");
        // Cannot connect, return blue
        return pixels.Color( 0, 0, 255 );
    }

    // We now create a URI for the request
    String strUrl = "/spaceAPI/";

    Serial.print( "Requesting URL: " );
    Serial.println( strUrl );
  
    // This will send the request to the server
    client.print( String("GET ") + strUrl + " HTTP/1.0\r\n" +
                  "Host: " + strServer + "\r\n\r\n" );

    // Read until we have a valid character
    character = client.read( );
    while( character == 255 )
    {
        delay( 10 );
        character = client.read( );
    }

    // Read data
    while( client.connected() && !jsonreader.finished( ) )
    {
        switch ( character )
        {
            case '\n':
            case '\r':
                nNewLines++;
                break;

            default:
                if ( nNewLines < 4 )
                    nNewLines = 0;
                break;
        }

        if ( nNewLines >= 4 )
        {
            nPos++;
            Serial.print( character );
        }

        if ( nNewLines >= 4 && jsonreader.process_char( character ) != 0 )
        {
            Serial.print( "Invalid JSON! chat at " );
            Serial.print( nPos );
            Serial.print( "=>" );
            Serial.print( character );
            Serial.print( " (0x" );
            Serial.print( character, HEX );
            Serial.print( ")" );
            return pixels.Color( 255, 0, 255 );
        }

        character = client.read( );
    }

    // Space open, return green
    if ( strcmpi( jsonreader.results[ 0 ], "true" ) )
        return pixels.Color( 0, 255, 0 );

    // Space closed, return green
    if ( strcmpi( jsonreader.results[ 0 ], "false" ) )
        return pixels.Color( 255, 0, 0 );

    // Unknown state, return yellow
    return pixels.Color( 255, 150, 0 );
}
#endif
